/*
$Id: ebu_misc.c,v 1.5 2006/01/02 18:24:03 rasc Exp $


 DVBSNOOP

 a dvb sniffer  and mpeg2 stream analyzer tool
 http://dvbsnoop.sourceforge.net/

 (c) 2001-2006   Rainer.Scherg@gmx.de  (rasc)



 -- misc. routines for
 -- EBU data (see EN 300 472, EN 301 775, and some more)
 -- Teletext, VPS, WSS, closed caption, etc.




$Log: ebu_misc.c,v $
Revision 1.5  2006/01/02 18:24:03  rasc
just update copyright and prepare for a new public tar ball

Revision 1.4  2005/01/17 19:41:22  rasc
Bugfix: data broadcast descriptor (tnx to Sergio SAGLIOCCO, SecureLAB)

Revision 1.3  2004/11/04 19:21:11  rasc
Fixes and changes on "premiere.de" private sections
Cleaning up "premiere.de" private descriptors (should be final now)

Revision 1.2  2004/02/05 10:30:56  rasc
no message

Revision 1.1  2004/02/04 22:36:27  rasc
more EBU/teletext stuff






*/




#include "dvbsnoop.h"
#include "ebu_misc.h"
#include "strings/dvb_str.h"
#include "misc/helper.h"
#include "misc/hexprint.h"
#include "misc/output.h"





// -- some tables and some routines basics
// -- are from dvbtext (c) Dave Chapman  and 
// -- originally from Ralph Metzler's vbidecode package. 
// -- other basic considerations are transfered from tuxtxt (c) LazyT



//
// Inverttab 
//  -- LSb... MSb  -> MSb...LSb
//  -- shift transmission order of
//  -- bits  (12345678) ->  cpu order (87654321)
//

static u_char invtab[256] = {
  0x00, 0x80, 0x40, 0xc0, 0x20, 0xa0, 0x60, 0xe0, 
  0x10, 0x90, 0x50, 0xd0, 0x30, 0xb0, 0x70, 0xf0, 
  0x08, 0x88, 0x48, 0xc8, 0x28, 0xa8, 0x68, 0xe8, 
  0x18, 0x98, 0x58, 0xd8, 0x38, 0xb8, 0x78, 0xf8, 
  0x04, 0x84, 0x44, 0xc4, 0x24, 0xa4, 0x64, 0xe4, 
  0x14, 0x94, 0x54, 0xd4, 0x34, 0xb4, 0x74, 0xf4, 
  0x0c, 0x8c, 0x4c, 0xcc, 0x2c, 0xac, 0x6c, 0xec, 
  0x1c, 0x9c, 0x5c, 0xdc, 0x3c, 0xbc, 0x7c, 0xfc, 
  0x02, 0x82, 0x42, 0xc2, 0x22, 0xa2, 0x62, 0xe2, 
  0x12, 0x92, 0x52, 0xd2, 0x32, 0xb2, 0x72, 0xf2, 
  0x0a, 0x8a, 0x4a, 0xca, 0x2a, 0xaa, 0x6a, 0xea, 
  0x1a, 0x9a, 0x5a, 0xda, 0x3a, 0xba, 0x7a, 0xfa, 
  0x06, 0x86, 0x46, 0xc6, 0x26, 0xa6, 0x66, 0xe6, 
  0x16, 0x96, 0x56, 0xd6, 0x36, 0xb6, 0x76, 0xf6, 
  0x0e, 0x8e, 0x4e, 0xce, 0x2e, 0xae, 0x6e, 0xee, 
  0x1e, 0x9e, 0x5e, 0xde, 0x3e, 0xbe, 0x7e, 0xfe, 
  0x01, 0x81, 0x41, 0xc1, 0x21, 0xa1, 0x61, 0xe1, 
  0x11, 0x91, 0x51, 0xd1, 0x31, 0xb1, 0x71, 0xf1, 
  0x09, 0x89, 0x49, 0xc9, 0x29, 0xa9, 0x69, 0xe9, 
  0x19, 0x99, 0x59, 0xd9, 0x39, 0xb9, 0x79, 0xf9, 
  0x05, 0x85, 0x45, 0xc5, 0x25, 0xa5, 0x65, 0xe5, 
  0x15, 0x95, 0x55, 0xd5, 0x35, 0xb5, 0x75, 0xf5, 
  0x0d, 0x8d, 0x4d, 0xcd, 0x2d, 0xad, 0x6d, 0xed, 
  0x1d, 0x9d, 0x5d, 0xdd, 0x3d, 0xbd, 0x7d, 0xfd, 
  0x03, 0x83, 0x43, 0xc3, 0x23, 0xa3, 0x63, 0xe3, 
  0x13, 0x93, 0x53, 0xd3, 0x33, 0xb3, 0x73, 0xf3, 
  0x0b, 0x8b, 0x4b, 0xcb, 0x2b, 0xab, 0x6b, 0xeb, 
  0x1b, 0x9b, 0x5b, 0xdb, 0x3b, 0xbb, 0x7b, 0xfb, 
  0x07, 0x87, 0x47, 0xc7, 0x27, 0xa7, 0x67, 0xe7, 
  0x17, 0x97, 0x57, 0xd7, 0x37, 0xb7, 0x77, 0xf7, 
  0x0f, 0x8f, 0x4f, 0xcf, 0x2f, 0xaf, 0x6f, 0xef, 
  0x1f, 0x9f, 0x5f, 0xdf, 0x3f, 0xbf, 0x7f, 0xff, 
};



//
// -- Invert Character/Buffer (LSb/MSb)
// -- bitreverse character/buffer (per char)
//

u_char invertChar (u_char *b)
{
   return invtab[ *b ];
}


void invertBuffer (u_char *b, int len)
{
  int i;

  for (i=0; i<len; i++) {
	*(b+i) = invtab[ *(b+i) ];
  }
}








// -- reset parity bit 
// -- buffer needs to be "normalized" before.
// 
//  Bit 1  2  3  4  5  6  7  8  (transmission order)
//      D1 D2 D3 D4 D5 D6 D7 P
// 

void unParityTeletextData (u_char *b, int len)
{
  int i;

  for (i=0; i<len; i++) {
	*(b+i) = *(b+i) & 0x7F; 
  }

}





// -- Hamming 8/4 table
//
//  Bit 1  2  3  4  5  6  7  8    (transmission order)
//      P1 D1 P2 D2 P3 D3 P4 D4
//
//      P = Control/Parity bit,  D = Data bit
//

static u_char unham84tab[256] = {
  0x01, 0xff, 0x01, 0x01, 0xff, 0x00, 0x01, 0xff, 
  0xff, 0x02, 0x01, 0xff, 0x0a, 0xff, 0xff, 0x07, 
  0xff, 0x00, 0x01, 0xff, 0x00, 0x80, 0xff, 0x00, 
  0x06, 0xff, 0xff, 0x0b, 0xff, 0x00, 0x03, 0xff, 
  0xff, 0x0c, 0x01, 0xff, 0x04, 0xff, 0xff, 0x07, 
  0x06, 0xff, 0xff, 0x07, 0xff, 0x07, 0x07, 0x87, 
  0x06, 0xff, 0xff, 0x05, 0xff, 0x00, 0x0d, 0xff, 
  0x86, 0x06, 0x06, 0xff, 0x06, 0xff, 0xff, 0x07, 
  0xff, 0x02, 0x01, 0xff, 0x04, 0xff, 0xff, 0x09, 
  0x02, 0x82, 0xff, 0x02, 0xff, 0x02, 0x03, 0xff, 
  0x08, 0xff, 0xff, 0x05, 0xff, 0x00, 0x03, 0xff, 
  0xff, 0x02, 0x03, 0xff, 0x03, 0xff, 0x83, 0x03, 
  0x04, 0xff, 0xff, 0x05, 0x84, 0x04, 0x04, 0xff, 
  0xff, 0x02, 0x0f, 0xff, 0x04, 0xff, 0xff, 0x07, 
  0xff, 0x05, 0x05, 0x85, 0x04, 0xff, 0xff, 0x05, 
  0x06, 0xff, 0xff, 0x05, 0xff, 0x0e, 0x03, 0xff, 
  0xff, 0x0c, 0x01, 0xff, 0x0a, 0xff, 0xff, 0x09, 
  0x0a, 0xff, 0xff, 0x0b, 0x8a, 0x0a, 0x0a, 0xff, 
  0x08, 0xff, 0xff, 0x0b, 0xff, 0x00, 0x0d, 0xff, 
  0xff, 0x0b, 0x0b, 0x8b, 0x0a, 0xff, 0xff, 0x0b, 
  0x0c, 0x8c, 0xff, 0x0c, 0xff, 0x0c, 0x0d, 0xff, 
  0xff, 0x0c, 0x0f, 0xff, 0x0a, 0xff, 0xff, 0x07, 
  0xff, 0x0c, 0x0d, 0xff, 0x0d, 0xff, 0x8d, 0x0d, 
  0x06, 0xff, 0xff, 0x0b, 0xff, 0x0e, 0x0d, 0xff, 
  0x08, 0xff, 0xff, 0x09, 0xff, 0x09, 0x09, 0x89, 
  0xff, 0x02, 0x0f, 0xff, 0x0a, 0xff, 0xff, 0x09, 
  0x88, 0x08, 0x08, 0xff, 0x08, 0xff, 0xff, 0x09, 
  0x08, 0xff, 0xff, 0x0b, 0xff, 0x0e, 0x03, 0xff, 
  0xff, 0x0c, 0x0f, 0xff, 0x04, 0xff, 0xff, 0x09, 
  0x0f, 0xff, 0x8f, 0x0f, 0xff, 0x0e, 0x0f, 0xff, 
  0x08, 0xff, 0xff, 0x05, 0xff, 0x0e, 0x0d, 0xff, 
  0xff, 0x0e, 0x0f, 0xff, 0x0e, 0x8e, 0xff, 0x0e, 
};





// -- simple unham Word 8_4  (2 bytes -> 8 bit)
// -- input byte order:  LSB, MSB
// FROM vbidecode

u_char unhamW84 (u_char lsb, u_char msb)
{
  u_char c1,c2;
  
  c1=unham84tab[lsb];
  c2=unham84tab[msb];
  //  if ((c1 | c2) & 0x40)  bad ham!
  return (c2 << 4) | (c1 & 0x0f);
}


// -- simple unham Byte 8_4  (1 bytes -> 4 bit)
u_char unhamB84 (u_char c)
{
  return unham84tab[c];
}




//
// -- simple unham Triplet 24_18  (3 bytes -> 18 bit)
// -- teletext data needs to be 'normalized'
//
// -- Hamming Code (24:18)
// 
//    Byte N                    Byte N + 1                  Byte N + 2
//bit 1  2  3  4  5  6  7  8  | 9  10 11 12 13 14  15  16 | 17  18  19  20  21  22  23  24
//    P1 P2 D1 P3 D2 D3 D4 P4 | D5 D6 D7 D8 D9 D10 D11 P5 | D12 D13 D14 D15 D16 D17 D18 P6
//
// P = Control/Parity bit,  D = Data bit
//
// because we are on "digital TV", I will keep this simple,
// we don't need to mess with parity bits. TS-Packets should do the integrity
// protection... (hopefully)
//

u_long unhamT24_18 (u_char lsb, u_char msb1, u_char msb2)
{
   u_long  v; 

	// Hamming Code (24:18)
   v = 0;
   v |= (lsb & 0x04) >> 2;  	// D1
   v |= (lsb & 0x70) >> 3;  	// D2..D4
   v |= ((msb1 & 0x7F ) << 8);	// D5..D11
   v |= ((msb2 & 0x7F ) << 16);	// D12..D18

   // $$$ TODO parity check, etc.
   return v;
}











// -- output EBU reserved, field_parity, line_offset
// -- length: 1 byte

void ebu_rfl_out (int v, u_char *b)
{
   	outBit_Sx_NL (6,"reserved: ",		b, 0, 2);
   	outBit_Sx_NL (v,"field_parity: ",	b, 2, 1);
   	outBit_Sx_NL (v,"line_offset: ",	b, 3, 5);
}












